LUABINS_NIL = 0x2D
LUABINS_FALSE = 0x30
LUABINS_TRUE = 0x31
LUABINS_NUMBER = 0x4E
LUABINS_STRING = 0x53
LUABINS_TABLE = 0x54

LUA_SHORT_SHORT_INT_LENGTH = 1
LUA_INT_LENGTH = 4
LUA_SIZE_T_LENGTH = 4
LUA_NUMBER_LENGTH = 8

LUA_BYTEORDER = "little"
LUA_STR_ENCODING = "utf-8"
